// ----------------------------------------------------------------------------
// Irrlicht Wrapper for Imperative Languages - Freebasic Examples
// Frank Dodd (2006)
// ----------------------------------------------------------------------------
// Example 09 : Skybox
// This example adds a skybox around the whole scene that makes a backdrop
// for your scene to be rendered against
// ----------------------------------------------------------------------------

program Example09_Skybox;

// ////////////////////////////////////////////////////////////////////////////
// Includes for extension libraries
uses IrrlichtWrapper;

// ////////////////////////////////////////////////////////////////////////////
// global variables

// irrlicht objects
var SkyBox: irr_node;
    Camera: irr_camera;


begin

// start the irrlicht interface
IrrStart( IRR_EDT_OPENGL, 640, 480, IRR_BITS_PER_PIXEL_32, 
        IRR_WINDOWED, IRR_SHADOWS, IRR_IGNORE_EVENTS );

// send the window caption
IrrSetWindowCaption( 'Example 09: Skybox' );

// the skybox is a simple hollow cube that surrounds the whole scene. textures
// are applied to all of the six sides of the cube creating an image around the
// entire scene instead of simply the color of the blank canvas
// here we load the textures as parameters of the skybox command (they could
// of course be loaded seperatly and assigned to irr_texture variables
SkyBox := IrrAddSkyBoxToScene( 
        IrrGetTexture('./media/irrlicht2_up.jpg'),
        IrrGetTexture('./media/irrlicht2_dn.jpg'),
        IrrGetTexture('./media/irrlicht2_rt.jpg'),
        IrrGetTexture('./media/irrlicht2_lf.jpg'),
        IrrGetTexture('./media/irrlicht2_ft.jpg'),
        IrrGetTexture('./media/irrlicht2_bk.jpg'));

// we add a first person perspective camera to the scene so you can look about
Camera := IrrAddFPSCamera;

// hide the mouse pointer
IrrHideMouse;

// -----------------------------------------------------------------------------
// while the irrlicht environment is still running
  while IrrRunning = 1 do
  begin
    // begin the scene, erasing the canvas with sky-blue before rendering
    IrrBeginScene( 240, 255, 255 );

    // draw the scene
    IrrDrawScene;

    // end drawing the scene and render it
    IrrEndScene;
  end;

// -----------------------------------------------------------------------------
// Stop the irrlicht engine and release resources
IrrStop;

end.