unit IrrlichtWrapper;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}

{$MINENUMSIZE 4}

interface

const
  {$IFDEF LINUX}
    DLLNAME = 'libIrrlichtWrapper.so.0.6.2';
  {$ELSE}
    DLLNAME = 'IrrlichtWrapper.dll';
  {$ENDIF}

  ////////////////////////////////////////////////////////////////////////////
  // Constant Definitions

  // No value, usually used when you do not wish to pass an object to a call
  IRR_NO_OBJECT = 0;

  // Parenting of nodes
  IRR_NO_PARENT = 0;

  // Flag definition
  IRR_OFF = 0;
  IRR_ON = 1;

  // Input definitions
  IRR_KEY_UP = 0;
  IRR_KEY_DOWN = 1;

  // The number of bits per pixel in the display
  IRR_BITS_PER_PIXEL_16 = 16;
  IRR_BITS_PER_PIXEL_32 = 32;

  // System Device definitions
  IRR_WINDOWED = 0;
  IRR_FULLSCREEN = 1;

  IRR_NO_SHADOWS = 0;
  IRR_SHADOWS = 1;

  IRR_IGNORE_EVENTS = 0;
  IRR_CAPTURE_EVENTS = 1;

  IRR_LINEAR_FOG = 0;
  IRR_EXPONENTIAL_FOG = 1;

  IRR_VERTICAL_SYNC_OFF = 0;
  IRR_VERTICAL_SYNC_ON = 1;

  // Filing system definitions
  IRR_USE_CASE = 0;
  IRR_IGNORE_CASE = 1;
  IRR_USE_PATHS = 0;
  IRR_IGNORE_PATHS = 1;

  // 2D image definitions
  IRR_IGNORE_ALPHA = 0;
  IRR_USE_ALPHA = 1;

  // Node definitions
  IRR_INVISIBLE = 0;
  IRR_VISIBLE = 1;

  IRR_ONE_SHOT = 0;
  IRR_LOOP = 1;

  // Particle definitions
  IRR_NO_EMITTER = 0;
  IRR_DEFAULT_EMITTER = 1;

  IRR_ATTRACT = 1;
  IRR_REPEL = 0;

  // GUI Interface definitions
  IRR_GUI_NO_BORDER = 0;
  IRR_GUI_BORDER = 1;
  IRR_GUI_NO_WRAP = 0;
  IRR_GUI_WRAP = 1;

  // Spherical Terrain Faces
  IRR_TOP_FACE = 0;
  IRR_FRONT_FACE = 1;
  IRR_BACK_FACE = 2;
  IRR_LEFT_FACE = 3;
  IRR_RIGHT_FACE = 4;
  IRR_BOTTOM_FACE = 5;
  
  TOP_EDGE = 0;
  BOTTOM_EDGE = 1;
  LEFT_EDGE = 2;
  RIGHT_EDGE = 3;
  
type
// Enumerated Type Definitions

// Rendering Device Types
IRR_DEVICE_TYPES = (
    IRR_EDT_NULL = 0,            // a NULL device with no display
    IRR_EDT_SOFTWARE,            // Irrlichts default software renderer
    IRR_EDT_SOFTWARE2,           // An improved quality software renderer
    IRR_EDT_OPENGL,              // hardware accelerated OpenGL renderer
    IRR_EDT_DIRECT3D8,           // hardware accelerated DirectX 8 renderer
    IRR_EDT_DIRECT3D9            // hardware accelerated DirectX 9 renderer
);

// Material Flags
IRR_MATERIAL_FLAGS = (
    IRR_EMF_WIREFRAME = 0,       // render as wireframe outline
    IRR_EMF_GOURAUD_SHADING,     // render smoothly across polygons
    IRR_EMF_LIGHTING,            // material is effected by lighting
    IRR_EMF_ZBUFFER,             // enable z buffer
    IRR_EMF_ZWRITE_ENABLE,       // can write as well as read z buffer
    IRR_EMF_BACK_FACE_CULLING,   // cull polygons facing away
    IRR_EMF_BILINEAR_FILTER,     // enable bilinear filtering
    IRR_EMF_TRILINEAR_FILTER,    // enable trilinear filtering
    IRR_EMF_ANISOTROPIC_FILTER,  // reduce blur in distant textures
    IRR_EMF_FOG_ENABLE,          // enable fogging in the distance
    IRR_EMF_NORMALIZE_NORMALS   // use when scaling dynamically lighted models
); 

// Material Types
IRR_MATERIAL_TYPES = (
    IRR_EMT_SOLID = 0,           // Standard solid r);ering uses one texture
    IRR_EMT_SOLID_2_LAYER,       // 2 blended textures using vertex alpha value
    IRR_EMT_LIGHTMAP,            // 2 textures: 0=color 1=lighting level and ignores vertex lighting
    IRR_EMT_LIGHTMAP_ADD,        // ... as above but adds levels instead of modulating between them
    IRR_EMT_LIGHTMAP_M2,         // ... as above but color levels are multiplied by 2 for brightening
    IRR_EMT_LIGHTMAP_M4,         // ... as above but color leels are multiplied by 4 for brightening
    IRR_EMT_LIGHTMAP_LIGHTING,   // 2 textures: 0=color 1=lighting level but supports dynamic lighting
    IRR_EMT_LIGHTMAP_LIGHTING_M2,    // ... as above but color levels are multiplied by 2 for brightening
    IRR_EMT_LIGHTMAP_LIGHTING_M4,    // ... as above but color leels are multiplied by 4 for brightening
    IRR_EMT_DETAIL_MAP,          // 2 blended textures: the first is a color map the second at a different scale adds and subtracts from the color to add detail
    IRR_EMT_SPHERE_MAP,          // makes the material appear reflective
    IRR_EMT_REFLECTION_2_LAYER,  // a reflective material blended with a color texture
    IRR_EMT_TRANSPARENT_ADD_COLOR,   // a transparency effect that simply adds a color texture to the background. the darker the color the more transparent it is.
    IRR_EMT_TRANSPARENT_ALPHA_CHANNEL,   // a transparency effect that uses the color textures alpha as a transparency level
    IRR_EMT_TRANSPARENT_ALPHA_CHANNEL_REF,   // a transparency effect that uses the color textures alpha, the pixel is only drawn if the alpha is > 127. this is a fast effect that does not blur edges and is ideal for leaves & grass etc.
    IRR_EMT_TRANSPARENT_VERTEX_ALPHA,    // a transparency effect that uses the vertex alpha value
    IRR_EMT_TRANSPARENT_REFLECTION_2_LAYER,  // a transparent & reflecting effect. the first texture is a reflection map, the second a color map. transparency is from vertex alpha
    IRR_EMT_NORMAL_MAP_SOLID,    // A solid normal map renderer. First texture is color, second is normal map. Only use nodes added with IrrAddStaticMeshForNormalMappingToScene. Only supports nearest two lights. Requires vertex and pixel shaders 1.1
    IRR_EMT_NORMAL_MAP_TRANSPARENT_ADD_COLOR,    // ... as above only with a transparency effect that simply adds the color to the background. the darker the color the more transparent it is.
    IRR_EMT_NORMAL_MAP_TRANSPARENT_VERTEX_ALPHA, // ... as above only with a transparency effect that uses the vertex alpha value
    IRR_EMT_PARALLAX_MAP_SOLID,  // similar to the solid normal map but more realistic providing virtual displacement of the surface. Uses the alpha channel of the normal map for height field displacement. Requires vertex shader 1.1 and pixel shader 1.4.
    IRR_EMT_PARALLAX_MAP_TRANSPARENT_ADD_COLOR,  // ... as above only with a transparency effect that simply adds the color to the background. the darker the color the more transparent it is.
    IRR_EMT_PARALLAX_MAP_TRANSPARENT_VERTEX_ALPHA,   // ... as above only wiht a transparency effect that uses the vertex alpha value
    IRR_EMT_ONE_TEXTURE_BLEND,
    IRR_EMT_FOUR_DETAIL_MAP,  // 4 grayscale images in the channels of the first texture mixed with the vertex channels as alpha images
    IRR_EMT_TRANSPARENT_ADD_ALPHA_CHANNEL_REF, // as above only it adds the texture color rather than replacing it
    IRR_EMT_FORCE_32BIT = $7fffffff
); 

// Mouse events
IRR_MOUSE_EVENTS = (
    IRR_EMIE_LMOUSE_PRESSED_DOWN = 0,
    IRR_EMIE_RMOUSE_PRESSED_DOWN,
    IRR_EMIE_MMOUSE_PRESSED_DOWN,
    IRR_EMIE_LMOUSE_LEFT_UP,
    IRR_EMIE_RMOUSE_LEFT_UP,
    IRR_EMIE_MMOUSE_LEFT_UP,
    IRR_EMIE_MOUSE_MOVED,
    IRR_EMIE_MOUSE_WHEEL
); 

// MD2 Animation sequences
IRR_MD2_ANIM_SEQUENCES = (
    IRR_EMAT_STAND = 0,
    IRR_EMAT_RUN,
    IRR_EMAT_ATTACK,
    IRR_EMAT_PAIN_A,
    IRR_EMAT_PAIN_B,
    IRR_EMAT_PAIN_C,
    IRR_EMAT_JUMP,
    IRR_EMAT_FLIP,
    IRR_EMAT_SALUTE,
    IRR_EMAT_FALLBACK,
    IRR_EMAT_WAVE,
    IRR_EMAT_POINT,
    IRR_EMAT_CROUCH_STAND,
    IRR_EMAT_CROUCH_WALK,
    IRR_EMAT_CROUCH_ATTACK,
    IRR_EMAT_CROUCH_PAIN,
    IRR_EMAT_CROUCH_DEATH,
    IRR_EMAT_DEATH_FALLBACK,
    IRR_EMAT_DEATH_FALLFORWARD,
    IRR_EMAT_DEATH_FALLBACKSLOW,
    IRR_EMAT_BOOM
); 

// For the FPS camera
EKEY_ACTION = (
    EKA_MOVE_FORWARD = 0,
    EKA_MOVE_BACKWARD,
    EKA_STRAFE_LEFT,
    EKA_STRAFE_RIGHT,
    EKA_JUMP_UP,
    EKA_COUNT,
    EKA_FORCE_32BIT = $7fffffff
); 

// Light types
E_LIGHT_TYPE = (
    ELT_POINT = 0,
    ELT_SPOT,
    ELT_DIRECTIONAL
); 

// Joint animation modes
IRR_JOINT_MODE = (
    IRR_JOINT_MODE_NONE = 0,
    IRR_JOINT_MODE_READ,
    IRR_JOINT_MODE_CONTROL
); 

// Pre-programmed shader constants
IRR_SHADER_CONSTANTS = (
    IRR_NO_PRESET = 0,
    IRR_INVERSE_WORLD,
    IRR_WORLD_VIEW_PROJECTION,
    IRR_CAMERA_POSITION,
    IRR_TRANSPOSED_WORLD
); 

// Vertex shader program versions
IRR_VERTEX_SHADER_VERSION = (
    EVST_VS_1_1 = 0,
    EVST_VS_2_0,
    EVST_VS_2_a,
    EVST_VS_3_0
); 

// Pixel shader program versions
IRR_PIXEL_SHADER_VERSION = (
    EPST_PS_1_1 = 0,
    EPST_PS_1_2,
    EPST_PS_1_3,
    EPST_PS_1_4,
    EPST_PS_2_0,
    EPST_PS_2_a,
    EPST_PS_2_b,
    EPST_PS_3_0
); 

// eration for querying features of the video driver. 
IRR_VIDEO_FEATURE_QUERY = (
    EVDF_RENDER_TO_TARGET = 0,   // Is driver able to render to a surface?  
    EVDF_HARDWARE_TL,            // Is hardeware transform and lighting supported?  
    EVDF_MULTITEXTURE,           // Are multiple textures per material possible?  
    EVDF_BILINEAR_FILTER,        // Is driver able to r);er with a bilinear filter applied?  
    EVDF_MIP_MAP,                // Can the driver handle mip maps?  
    EVDF_MIP_MAP_AUTO_UPDATE,    // Can the driver update mip maps automatically?  
    EVDF_STENCIL_BUFFER,         // Are stencilbuffers switched on and does the device support stencil buffers?  
    EVDF_VERTEX_SHADER_1_1,      // Is Vertex Shader 1.1 supported?  
    EVDF_VERTEX_SHADER_2_0,      // Is Vertex Shader 2.0 supported?  
    EVDF_VERTEX_SHADER_3_0,      // Is Vertex Shader 3.0 supported?  
    EVDF_PIXEL_SHADER_1_1,       // Is Pixel Shader 1.1 supported?  
    EVDF_PIXEL_SHADER_1_2,       // Is Pixel Shader 1.2 supported?  
    EVDF_PIXEL_SHADER_1_3,       // Is Pixel Shader 1.3 supported?  
    EVDF_PIXEL_SHADER_1_4,       // Is Pixel Shader 1.4 supported?  
    EVDF_PIXEL_SHADER_2_0,       // Is Pixel Shader 2.0 supported?  
    EVDF_PIXEL_SHADER_3_0,       // Is Pixel Shader 3.0 supported?  
    EVDF_ARB_VERTEX_PROGRAM_1,   // Are ARB vertex programs v1.0 supported?  
    EVDF_ARB_FRAGMENT_PROGRAM_1, // Are ARB fragment programs v1.0 supported?  
    EVDF_ARB_GLSL,               // Is GLSL supported?  
    EVDF_HLSL,                   // Is HLSL supported?  
    EVDF_TEXTURE_NPOT,           // Are non-power-of-two textures supported?  
    EVDF_FRAMEBUFFER_OBJECT     // Are framebuffer objects supported?
); 

// eration flags defining the file format supported loading and saving
IRR_MESH_FILE_FORMAT = (
    EMWT_IRR_MESH = 0,            // Irrlicht Native mesh writer, for static .irrmesh files.  
    EMWT_COLLADA,                 // COLLADA mesh writer for .dae and .xml files.  
    EMWT_STL                     // STL mesh writer for .stl files.  
); 

// eration flags defining the size of a patch in the terrain
IRR_TERRAIN_PATCH_SIZE = (
    ETPS_9 = 9,                    // patch size of 9, at most, use 4 levels of detail with this patch size.  
    ETPS_17 = 17,                  // patch size of 17, at most, use 5 levels of detail with this patch size.  
    ETPS_33 = 33,                  // patch size of 33, at most, use 6 levels of detail with this patch size.  
    ETPS_65 = 65,                  // patch size of 65, at most, use 7 levels of detail with this patch size.  
    ETPS_129 = 129                // patch size of 129, at most, use 8 levels of detail with this patch size. 
); 

// eration flags telling the video driver in which format textures should be created. 
IRR_TEXTURE_CREATION_FLAG = (
    ETCF_ALWAYS_16_BIT = 0,      // Forces the driver to create 16 bit textures always, indep);ent of which format the file on disk has. When choosing this you may loose some color detail, but gain much speed and memory. 16 bit textures can be transferred twice as fast as 32 bit textures and only use half of the space in memory. When using this flag, it does not make sense to use the flags ETCF_ALWAYS_32_BIT, ETCF_OPTIMIZED_FOR_QUALITY, or ETCF_OPTIMIZED_FOR_SPEED at the same time.  
    ETCF_ALWAYS_32_BIT,          // Forces the driver to create 32 bit textures always, indep);ent of which format the file on disk has. Please note that some drivers (like the software device) will ignore this, because they are only able to create and use 16 bit textures. When using this flag, it does not make sense to use the flags ETCF_ALWAYS_16_BIT, ETCF_OPTIMIZED_FOR_QUALITY, or ETCF_OPTIMIZED_FOR_SPEED at the same time.  
    ETCF_OPTIMIZED_FOR_QUALITY,  // Lets the driver decide in which format the textures are created and tries to make the textures look as good as possible. Usually it simply chooses the format in which the texture was stored on disk. When using this flag, it does not make sense to use the flags ETCF_ALWAYS_16_BIT, ETCF_ALWAYS_32_BIT, or ETCF_OPTIMIZED_FOR_SPEED at the same time.  
    ETCF_OPTIMIZED_FOR_SPEED,    // Lets the driver decide in which format the textures are created and tries to create them maximizing r);er speed. When using this flag, it does not make sense to use the flags ETCF_ALWAYS_16_BIT, ETCF_ALWAYS_32_BIT, or ETCF_OPTIMIZED_FOR_QUALITY, at the same time.  
    ETCF_CREATE_MIP_MAPS,        // Automatically creates mip map levels for the textures.  
    ETCF_NO_ALPHA_CHANNEL       // Discard any alpha layer and use non-alpha color format.  
); 

// A color format specifies how color information is stored
IRR_COLOR_FORMAT = (
    ECF_A1R5G5B5,        // 16 bit color format used by the software driver, and thus preferred by all other irrlicht engine video drivers. There are 5 bits for every color component, and a single bit is left for alpha information.  
    ECF_R5G6B5,          // Standard 16 bit color format.  
    ECF_R8G8B8,          // 24 bit color, no alpha channel, but 8 bit for red, green and blue.  
    ECF_A8R8G8B8        // Default 32 bit color format. 8 bits are used for every component: red, green, blue and alpha.  
); 

IRR_TEXTURE_BLEND = (
    BLEND_SCREEN = 0,
    BLEND_ADD,
    BLEND_SUBTRACT,
    BLEND_MULTIPLY,
    BLEND_DIVIDE
); 

IRR_DEBUG = (
    EDS_OFF = 0,
    EDS_BBOX = 1,
    EDS_NORMALS = 2,
    EDS_SKELETON = 4,
    EDS_MESH_WIRE_OVERLAY = 8,
    EDS_HALF_TRANSPARENCY = 16,
    EDS_BBOX_BUFFERS = 32,
    EDS_FULL = $ffffffff
); 

EKEY_CODE = (
    KEY_LBUTTON          = $01,  // Left mouse button  
    KEY_RBUTTON          = $02,  // Right mouse button  
    KEY_CANCEL           = $03,  // Control-break processing  
    KEY_MBUTTON          = $04,  // Middle mouse button (three-button mouse)  
    KEY_XBUTTON1         = $05,  // Windows 2000/XP: X1 mouse button 
    KEY_XBUTTON2         = $06,  // Windows 2000/XP: X2 mouse button 
    KEY_BACK             = $08,  // BACKSPACE key  
    KEY_TAB              = $09,  // Tab key  
    KEY_CLEAR            = $0C,  // Clear key  
    KEY_RETURN           = $0D,  // ENTER key  
    KEY_SHIFT            = $10,  // SHIFT key  
    KEY_CONTROL          = $11,  // CTRL key  
    KEY_MENU             = $12,  // ALT key  
    KEY_PAUSE            = $13,  // PAUSE key  
    KEY_CAPITAL          = $14,  // CAPS Lock key  
    KEY_KANA             = $15,  // IME Kana mode 
    KEY_HANGUEL          = $15,  // IME Hanguel mode (maintained For compatibility use KEY_HANGUL) 
    KEY_HANGUL           = $15,  // IME Hangul mode 
    KEY_JUNJA            = $17,  // IME Junja mode 
    KEY_FINAL            = $18,  // IME final mode 
    KEY_HANJA            = $19,  // IME Hanja mode 
    KEY_KANJI            = $19,  // IME Kanji mode 
    KEY_ESCAPE           = $1B,  // ESC key  
    KEY_CONVERT          = $1C,  // IME convert 
    KEY_NONCONVERT       = $1D,  // IME nonconvert 
    KEY_ACCEPT           = $1E,  // IME accept 
    KEY_MODECHANGE       = $1F,  // IME mode change request 
    KEY_SPACE            = $20,  // SPACEBAR  
    KEY_PRIOR            = $21,  // PAGE UP key  
    KEY_NEXT             = $22,  // PAGE DOWN key  
    KEY_END              = $23,  // End key  
    KEY_HOME             = $24,  // HOME key  
    KEY_LEFT             = $25,  // Left ARROW key  
    KEY_UP               = $26,  // UP ARROW key  
    KEY_RIGHT            = $27,  // Right ARROW key  
    KEY_DOWN             = $28,  // DOWN ARROW key  
    KEY_SELECT           = $29,  // Select key  
    KEY_PRINT            = $2A,  // Print key
    KEY_EXECUT           = $2B,  // EXECUTE key  
    KEY_SNAPSHOT         = $2C,  // Print Screen key  
    KEY_INSERT           = $2D,  // INS key  
    KEY_DELETE           = $2E,  // DEL key  
    KEY_HELP             = $2F,  // HELP key  
    KEY_KEY_0            = $30,  // 0 key  
    KEY_KEY_1            = $31,  // 1 key  
    KEY_KEY_2            = $32,  // 2 key  
    KEY_KEY_3            = $33,  // 3 key  
    KEY_KEY_4            = $34,  // 4 key  
    KEY_KEY_5            = $35,  // 5 key  
    KEY_KEY_6            = $36,  // 6 key  
    KEY_KEY_7            = $37,  // 7 key  
    KEY_KEY_8            = $38,  // 8 key  
    KEY_KEY_9            = $39,  // 9 key  
    KEY_KEY_A            = $41,  // A key  
    KEY_KEY_B            = $42,  // B key  
    KEY_KEY_C            = $43,  // C key  
    KEY_KEY_D            = $44,  // D key  
    KEY_KEY_E            = $45,  // E key  
    KEY_KEY_F            = $46,  // F key  
    KEY_KEY_G            = $47,  // G key  
    KEY_KEY_H            = $48,  // H key  
    KEY_KEY_I            = $49,  // I key  
    KEY_KEY_J            = $4A,  // J key  
    KEY_KEY_K            = $4B,  // K key  
    KEY_KEY_L            = $4C,  // L key  
    KEY_KEY_M            = $4D,  // M key  
    KEY_KEY_N            = $4E,  // N key  
    KEY_KEY_O            = $4F,  // O key  
    KEY_KEY_P            = $50,  // P key  
    KEY_KEY_Q            = $51,  // Q key  
    KEY_KEY_R            = $52,  // R key  
    KEY_KEY_S            = $53,  // S key  
    KEY_KEY_T            = $54,  // T key  
    KEY_KEY_U            = $55,  // U key  
    KEY_KEY_V            = $56,  // V key  
    KEY_KEY_W            = $57,  // W key  
    KEY_KEY_X            = $58,  // X key  
    KEY_KEY_Y            = $59,  // Y key  
    KEY_KEY_Z            = $5A,  // Z key  
    KEY_LWIN             = $5B,  // Left Windows key (Microsoft Natural keyboard)  
    KEY_RWIN             = $5C,  // Right Windows key (Natural keyboard)  
    KEY_APPS             = $5D,  // Applications key (Natural keyboard)  
    KEY_SLEEP            = $5F,  // Computer Sleep key 
    KEY_NUMPAD0          = $60,  // Numeric keypad 0 key  
    KEY_NUMPAD1          = $61,  // Numeric keypad 1 key  
    KEY_NUMPAD2          = $62,  // Numeric keypad 2 key  
    KEY_NUMPAD3          = $63,  // Numeric keypad 3 key  
    KEY_NUMPAD4          = $64,  // Numeric keypad 4 key  
    KEY_NUMPAD5          = $65,  // Numeric keypad 5 key  
    KEY_NUMPAD6          = $66,  // Numeric keypad 6 key  
    KEY_NUMPAD7          = $67,  // Numeric keypad 7 key  
    KEY_NUMPAD8          = $68,  // Numeric keypad 8 key  
    KEY_NUMPAD9          = $69,  // Numeric keypad 9 key  
    KEY_MULTIPLY         = $6A,  // Multiply key  
    KEY_ADD              = $6B,  // Add key  
    KEY_SEPARATOR        = $6C,  // Separator key  
    KEY_SUBTRACT         = $6D,  // Subtract key  
    KEY_DECIMAL          = $6E,  // Decimal key  
    KEY_DIVIDE           = $6F,  // Divide key  
    KEY_F1               = $70,  // F1 key  
    KEY_F2               = $71,  // F2 key  
    KEY_F3               = $72,  // F3 key  
    KEY_F4               = $73,  // F4 key  
    KEY_F5               = $74,  // F5 key  
    KEY_F6               = $75,  // F6 key  
    KEY_F7               = $76,  // F7 key  
    KEY_F8               = $77,  // F8 key  
    KEY_F9               = $78,  // F9 key  
    KEY_F10              = $79,  // F10 key  
    KEY_F11              = $7A,  // F11 key  
    KEY_F12              = $7B,  // F12 key  
    KEY_F13              = $7C,  // F13 key  
    KEY_F14              = $7D,  // F14 key  
    KEY_F15              = $7E,  // F15 key  
    KEY_F16              = $7F,  // F16 key  
    KEY_F17              = $80,  // F17 key  
    KEY_F18              = $81,  // F18 key  
    KEY_F19              = $82,  // F19 key  
    KEY_F20              = $83,  // F20 key  
    KEY_F21              = $84,  // F21 key  
    KEY_F22              = $85,  // F22 key  
    KEY_F23              = $86,  // F23 key  
    KEY_F24              = $87,  // F24 key  
    KEY_NUMLOCK          = $90,  // NUM Lock key  
    KEY_SCROLL           = $91,  // SCROLL Lock key  
    KEY_LSHIFT           = $A0,  // Left SHIFT key 
    KEY_RSHIFT           = $A1,  // Right SHIFT key 
    KEY_LCONTROL         = $A2,  // Left CONTROL key 
    KEY_RCONTROL         = $A3,  // Right CONTROL key 
    KEY_LMENU            = $A4,  // Left MENU key 
    KEY_RMENU            = $A5,  // Right MENU key 
    KEY_PLUS             = $BB,  // Plus Key   (+)
    KEY_COMMA            = $BC,  // Comma Key  (,)
    KEY_MINUS            = $BD,  // Minus Key  (-)
    KEY_PERIOD           = $BE,  // Period Key (.)
    KEY_ATTN             = $F6,  // Attn key 
    KEY_CRSEL            = $F7,  // CrSel key 
    KEY_EXSEL            = $F8,  // ExSel key 
    KEY_EREOF            = $F9,  // Erase Eof key 
    KEY_PLAY             = $FA,  // Play key 
    KEY_ZOOM             = $FB,  // Zoom key 
    KEY_PA1              = $FD,  // PA1 key 
    KEY_OEM_CLEAR        = $FE,  // Clear key 
    KEY_KEY_CODES_COUNT  = $FF  // this Is Not a key but the amount of keycodes there are.
);

//// ////////////////////////////////////////////////////////////////////////////
//// Type Definitions

// Irrlicht Keyboard and Mouse Event Structures
IRR_MOUSE_EVENT = record
    action: Cardinal;
    delta: Single;
    x: integer;
    y: integer;
end;
PIRR_MOUSE_EVENT = ^IRR_MOUSE_EVENT;

IRR_KEY_EVENT = record
    key: Cardinal;
    direction: Cardinal;
    flags: Cardinal;
end;
PIRR_KEY_EVENT = ^IRR_KEY_EVENT;

// Particle emitter properties
IRR_PARTICLE_EMITTER = record
    min_box_x: Single;     // The bounding box for the emitter
    min_box_y: Single;
    min_box_z: Single;

    max_box_x: Single;
    max_box_y: Single;
    max_box_z: Single;

    direction_x: Single;
    direction_y: Single;
    direction_z: Single;

    min_paritlcles_per_second: Cardinal;
    max_paritlcles_per_second: Cardinal;

    min_start_color_red: Integer;
    min_start_color_green: Integer;
    min_start_color_blue: Integer;

    max_start_color_red: Integer;
    max_start_color_green: Integer;
    max_start_color_blue: Integer;
    
    min_lifetime: Cardinal;
    max_lifetime: Cardinal;
    
    max_angle_degrees: Integer;
end;

// a vertex used in creating custom static mesh objects
IRR_VERT = record
    x: Single;             // The x position of the vertex
    y: Single;             // The y position of the vertex
    z: Single;             // The z position of the vertex
    normal_x: Single;      // The x normal of the vertex
    normal_y: Single;      // The y normal of the vertex
    normal_z: Single;      // The z normal of the vertex
    vcolor: Cardinal;      // The 32bit ARGB color of the vertex
    texture_x: Single;     // the x co-ordinate of the vertex on the texture (0 to 1)
    texture_y: Single;     // the y co-ordinate of the vertex on the texture (0 to 1)
end;
PIRR_VERT = ^IRR_VERT;

// a vector consisting of 3 float values
IRR_VECTOR = record
    x: Single;
    y: Single;
    z: Single;
end;

// the contents of a key event
SKeyMap = record
    Action: EKEY_ACTION;
    KeyCode: EKEY_CODE;
end;
PSKeyMap = ^SKeyMap;

// a material representing a new shader
IRR_SHADER = record
    // use the material  in a call to IrrSetNodeMaterial
    material_: Integer;
    // the following properties are reserved for internal use changing them can
    // result in crashes and memory leakage
    material_object: PCardinal;
    next_shader: PCardinal;
end;
PIRR_SHADER = ^IRR_SHADER;


// .............................................................................
// Irrlicht Object s - these are used to represent objects in the Irrlicht
// engine and their purpose is to warn the programmer of  errors
// (thanks to v1ctor for the advice on this correction)
irr_mesh = PCardinal;

irr_image = PCardinal;

irr_texture = PCardinal;

irr_material = PCardinal;

irr_node = PCardinal;

irr_camera = PCardinal;

irr_terrain = PCardinal;

irr_selector = PCardinal;

irr_particle_system = PCardinal;

irr_emitter = PCardinal;

irr_affector = PCardinal;

irr_animator = PCardinal;

irr_font = PCardinal;

irr_scene_node = PCardinal;

irr_model = record
    node: irr_node;
    camera: irr_camera;
    terrain: irr_terrain;
    particles: irr_particle_system;
end;

//////////////////////////////////////////////////////////////////////////////
// System Functions

// Start the Irrlicht interface
procedure IrrStart(device_type: IRR_DEVICE_TYPES; iwidth, iheight: Integer; bitsperpixel: LongInt; fullscreen, use_shadows, iCaptureMouse: Integer; vsync: Integer = IRR_OFF); cdecl; external DLLNAME;

// allow transparency to write to the z buffer
procedure IrrTransparentZWrite(); cdecl; external DLLNAME;

// determine if the Irrlicht viewport is running. 0 = No 1 = Yes
function IrrRunning(): Integer; cdecl; external DLLNAME;

// initialise the frame drawing cycle, erasing the canvas ready for drawing
procedure IrrBeginScene(red, green, blue: Integer); cdecl; external DLLNAME;

// draw the scene to the canvas
procedure IrrDrawScene(); cdecl; external DLLNAME;

// draw scene manager objects to a texture surface, the texture must have been
// created with a call to IrrCreateRenderTargetTexture
procedure IrrDrawSceneToTexture(texture: irr_texture); cdecl; external DLLNAME;

// draw the user interface to the canvas
procedure IrrDrawGUI(); cdecl; external DLLNAME;

// end all drawing operations and display the rendered frame
procedure IrrEndScene(); cdecl; external DLLNAME;

// Stop the Irrlicht engine freeing up its resources
procedure IrrStop(); cdecl; external DLLNAME;

// Query a feature of the video card, returns 1 if it is supported and 0 if it isnt
function IrrQueryFeature(Feature: IRR_VIDEO_FEATURE_QUERY): Cardinal; cdecl; external DLLNAME;

// get the current time in milliseconds
function IrrGetTime(): Cardinal; cdecl; external DLLNAME;

// get the current frame rate
function IrrGetFPS(): Integer; cdecl; external DLLNAME;

// get the number of primitives (mostly triangles) drawn in the last frame
function IrrGetPrimitivesDrawn(): Cardinal; cdecl; external DLLNAME;

// set the caption in the irrlicht window
procedure IrrSetWindowCaption(text: PWideChar); cdecl; external DLLNAME;

// make 32 bit representation of an Alpha, Red, Green, Blue color
function IrrMakeARGB(Alpha, Red, Green, Blue: Integer): Cardinal;


//// ////////////////////////////////////////////////////////////////////////////
//// Input Event Functions

// find out if there is a key event ready to be read. returns 1 if there is an
// event available (the event receiver must have been started when the system
// was initialised)
function IrrKeyEventAvailable(): Integer; cdecl; external DLLNAME;

// read a key event out
function IrrReadKeyEvent: PIRR_KEY_EVENT; cdecl; external DLLNAME;

// find out if there is a mouse event ready to be read. returns 1 if there is an
// event available (the event receiver must have been started when the system
// was initialised)
function IrrMouseEventAvailable(): Integer; cdecl; external DLLNAME;

// read a mouse event out
function IrrReadMouseEvent(): PIRR_MOUSE_EVENT; cdecl; external DLLNAME;

procedure IrrDisplayMouse(show_mouse: Cardinal); cdecl; external DLLNAME;
// show and hide the mouse pointer
procedure IrrHideMouse();
procedure IrrShowMouse();

// set the position of the mouse pointer and return the relative change in position
procedure IrrSetMousePosition(var x: Single; var y: Single); cdecl; external DLLNAME;


//// ////////////////////////////////////////////////////////////////////////////
//// Filing System Functions

// add zip archive to the filing system allowing you to load files straight out
// of the zip file. common pk3 files are simply zip files
procedure IrrAddZipFile(path: PChar; ignore_case, ignore_paths: Cardinal); cdecl; external DLLNAME;

// set irrlicht current working directory
procedure IrrChangeWorkingDirectory(path: PChar); cdecl; external DLLNAME;

// get irrlicht current working directory
function IrrGetWorkingDirectory(): PChar; cdecl; external DLLNAME;



//// ////////////////////////////////////////////////////////////////////////////
//// 2D Functions

// sets texture creation flags, use IRR_ON and IRR_OFF to set the flag value
procedure IrrSetTextureCreationFlag(flag: IRR_TEXTURE_CREATION_FLAG; value: Cardinal); cdecl; external DLLNAME;

// load an image (images do not use video memory and cannot texture objects
function IrrGetImage(path: PChar): irr_image; cdecl; external DLLNAME;

// load a texture object
function IrrGetTexture(path: PChar): irr_texture; cdecl; external DLLNAME;

// create a blank texture
function IrrCreateTexture(path: PChar; x, y: integer; format: IRR_COLOR_FORMAT): irr_texture; cdecl; external DLLNAME;

// create a blank image
function IrrCreateImage(x, y: integer; format: IRR_COLOR_FORMAT): irr_image; cdecl; external DLLNAME;

// create a texture that is suitable for the scene manager to use: a surface to
// which it can render its 3d object. each the dimentions must be a power of two
// for example 128x128 or 256x256
function IrrCreateRenderTargetTexture(x, y: integer ): irr_texture; cdecl; external DLLNAME;

// remove a texture freeing its resources
procedure IrrRemoveTexture(texture: irr_texture); cdecl; external DLLNAME;

// create a normal map from a grayscale heightmap texture
procedure IrrMakeNormalMapTexture(texture: irr_texture; amplitude: Single); cdecl; external DLLNAME;

// locks the texture and returns a pointer to the pixels
function IrrLockTexture(texture: irr_texture): PCardinal; cdecl; external DLLNAME;

// unlock the texture, presumably after it has been modified and recreate the mipmap levels
procedure IrrUnlockTexture(texture: irr_texture); cdecl; external DLLNAME;

// locks the image and returns a pointer to the pixels
function IrrLockImage(image: irr_image): PCardinal; cdecl; external DLLNAME;

// unlock the image, presumably after it has been modified
procedure IrrUnlockImage(image: irr_image); cdecl; external DLLNAME;

// blend the source texture into the destination texture to create a single texture
function IrrBlendTextures(destination_texture, source_texture: irr_texture; x_offset, y_offset, operation: Integer ): Integer; cdecl; external DLLNAME;

// color key the image making parts of it transparent
procedure IrrColorKeyTexture(texture: irr_texture; red, green, blue: Integer); cdecl; external DLLNAME;

// draw a 2d image to the display
procedure IrrDraw2DImage(texture: irr_texture; x, y: Integer); cdecl; external DLLNAME;

// draw an image to the screen from a specified rectangle in the source image
// this enables you to place lots of images into one texture for 2D drawing. in
// addition this function also uses image alpha channels
procedure IrrDraw2DImageElement(texture: irr_texture; x, y, source_tx, source_ty, source_bx, source_by, usealpha: Integer); cdecl; external DLLNAME;

// load a bitmap based font
function IrrGetFont(path: PChar): irr_font; cdecl; external DLLNAME;

// draw text to the display using a bitmap font
procedure Irr2DFontDraw(font: irr_font; text: PWideChar; top_x, top_y, bottom_x, bottom_y: Integer); cdecl; external DLLNAME;

// save a screenshot out to a file
procedure IrrSaveScreenShot(text: PChar); cdecl; external DLLNAME;



//// ////////////////////////////////////////////////////////////////////////////
//// Material and GPU Programming Functions

// Set how shiny the material is the higher the value the more defined the highlights
procedure IrrMaterialSetShininess(material: irr_material; shininess: Single); cdecl; external DLLNAME;

// The color of specular highlights on the object
procedure IrrMaterialSetSpecularColor(material: irr_material; Alpha, Red, Green, Blue: Cardinal); cdecl; external DLLNAME;

// The color of diffuse lighting on the object
procedure IrrMaterialSetDiffuseColor(material: irr_material; Alpha, Red, Green, Blue: Cardinal); cdecl; external DLLNAME;

// The color of ambient light reflected by the object
procedure IrrMaterialSetAmbientColor(material: irr_material; Alpha, Red, Green, Blue: Cardinal); cdecl; external DLLNAME;

// The color of light emitted by the object
procedure IrrMaterialSetEmissiveColor(material: irr_material; Alpha, Red, Green, Blue: Cardinal); cdecl; external DLLNAME;

// set material specific parameter
procedure IrrMaterialSetMaterialTypeParam(material: irr_material; value: Single); cdecl; external DLLNAME;


// Creates a Vertex shader constant that allows you to change the value of a
// constant inside a GLSL shader during the execution of the program simply
//:sign one of the preset constants to the constant name or attach the constant
// to an array of floats and change the constant simply by changing the values
// in your array
// Returns: 1 if the constant was sucessfully created
function IrrCreateNamedVertexShaderConstant(shader: PIRR_SHADER; const_name: PChar; const_preset: Integer; const_data: PSingle; data_count: Integer): Integer; cdecl; external DLLNAME;

// Creates a Pixel shader constant, the operation of the function is the same
//: that of the vertex shader constant above.
// Returns: 1 if a constant was sucessfully created
function IrrCreateNamedPixelShaderConstant(shader: PIRR_SHADER; const_name: PChar; const_preset: Integer; const_data: PSingle; data_count: Integer): Integer; cdecl; external DLLNAME;

// the same: IrrCreateNamedVertexShaderConstant but uses an address instead
// of a name for low level shaders
// Returns: 1 if a constant was sucessfully created
function IrrCreateAddressedVertexShaderConstant(shader: PIRR_SHADER; const_address, const_preset: Integer; const_data: PSingle; data_count: Integer): Integer; cdecl; external DLLNAME;

// the same: IrrCreateNamedPixelShaderConstant but uses an address instead
// of a name for low level shaders
// Returns: 1 if a constant was sucessfully created
function IrrCreateAddressedPixelShaderConstant(shader: PIRR_SHADER; const_address, const_preset: Integer; const_data: PSingle; data_count: Integer): Integer; cdecl; external DLLNAME;

// creates a new material using a high level shading language. these programs
// that run on the graphics card can greatly add to the realism of the output
// but only modern cards will support them. Currently only the HLSL/D3D9 and
// GLSL/OpenGL is supported.
// 
// vertex program:
//	String containing the source of the vertex shader program. This can be 0 if
//	no vertex program shall be used.  
// vertex_start_function:
//	Name of the entry function of the vertex shader program
// vertex_program_type:
//	Vertex shader version used to compile the GPU program
// pixel_program:
//	String containing the source of the pixel shader program. This can be 0 if
//	no pixel shader shall be used.  
// pixel_start_function:
//	Entry name of the function of the pixel shader program
// pixel_program_type:
//	Pixel shader version used to compile the GPU program
// baseMaterial:
//	Base material which renderstates will be used to shade the material.  
//
// Return:
//   Returns a type that contains a material_type number that can be used to
// shade nodes with this new material. If the shader could not be created it
// will return 0
//
function IrrAddHighLevelShaderMaterial(vertex_program, vertex_start_function: PChar; vertex_prog_type: Cardinal; pixel_program, pixel_start_function: PChar; pixel_prog_type, material_type: Cardinal ): PIRR_SHADER; cdecl; external DLLNAME;

// Identical to IrrAddHighLevelShaderMaterial only this varient attempts to load
// the GPU programs from files
function IrrAddHighLevelShaderMaterialFromFiles(vertex_program_filename, vertex_start_function: PChar; vertex_prog_type: Cardinal; pixel_program_filename, pixel_start_function: PChar; pixel_prog_type, material_type: Cardinal): PIRR_SHADER; cdecl; external DLLNAME;

// creates a new material using a high level shading language. these programs
// that run on the graphics card can greatly add to the realism of the output
// but only modern cards will support them. Currently only the HLSL/D3D9 and
// GLSL/OpenGL is supported.
// 
// vertex program:
//	String containing the source of the vertex shader program. This can be 0 if
//	no vertex program shall be used. For DX8 programs, the will always input
//	registers look like this: v0: position, v1: normal, v2: color, v3: texture
//	cooridnates, v4: texture coordinates 2 if available. For DX9 programs, you
//	can manually set the registers using the dcl_ statements.  
// pixel_program:
//	String containing the source of the pixel shader program. This can be 0 if
//	no pixel shader shall be used.  
// pixel_start_function:
//	Entry name of the function of the pixel shader program
// pixel_program_type:
//	Pixel shader version used to compile the GPU program
// baseMaterial:
//	Base material which renderstates will be used to shade the material.  
//
// Return:
//   Returns a type that contains a material_type number that can be used to
// shade nodes with this new material. If the shader could not be created it
// will return 0
//
function IrrAddShaderMaterial(vertex_program, pixel_program: PChar; material_type: Cardinal): PIRR_SHADER; cdecl; external DLLNAME;

// Identical to IrraddShaderMaterial only this varient attempts to load the GPU
// programs from file.
function IrrAddShaderMaterialFromFiles(vertex_program_filename, pixel_program_filename: PChar; material_type: Cardinal): PIRR_SHADER; cdecl; external DLLNAME;



//// ////////////////////////////////////////////////////////////////////////////
//// Scene Manager Functions

// load a mesh from a file
function IrrGetMesh(path: PChar): irr_mesh; cdecl; external DLLNAME;

// create a mesh from an array of verticies, an array of incidies to those
// verticies that connect them to form triangles and finally texture co-ordinates
// to map the verticies against a texture plane
function IrrCreateMesh(mesh_name: PChar; vertex_count: integer; varvertices: IRR_VERT; indices_count: integer; varindices: Word): irr_mesh; cdecl; external DLLNAME;

// adds a hill plane mesh to the mesh pool
Function IrrAddHillPlaneMesh(mesh_name: PChar; tileSizeX, tileSizeY: Single; tileCountX, tileCountY: Integer; material: PCardinal = 0; hillHeight: Single = 0; countHillsX: Single = 0; countHillsY: Single = 0; textureRepeatCountX: Single = 1; textureRepeatCountY: Single = 1): irr_mesh; cdecl; external DLLNAME;

// write the first frame of the supplied animated mesh out to a file using the specified file format
function IrrWriteMesh(mesh: irr_mesh; file_format: IRR_MESH_FILE_FORMAT; path: PChar): Cardinal; cdecl; external DLLNAME;

// remove a loaded mesh from the scene cache useful if you are dealing with large
// numbers of meshes that you dont want cached in memory at the same time (for
// example when swapping between BSP maps for levels
procedure IrrRemoveMesh(mesh: irr_mesh); cdecl; external DLLNAME;

// Clears all meshes that are held in the mesh cache but not used anywhere else.
procedure IrrClearUnusedMeshes(); cdecl; external DLLNAME;

// get the number of indicies in the mesh buffer
function IrrGetMeshIndexCount(mesh: irr_mesh; frame: Integer): Integer; cdecl; external DLLNAME;

// copy the indicies of a mesh into the supplied buffer, the caller must ensure
// that the buffer is big enough to store the data
procedure IrrGetMeshIndices(mesh: irr_mesh; frame: Integer; varindicies: Word); cdecl; external DLLNAME;

// copy the indicies in the supplied buffer into the mesh , the caller must
// ensure that the buffer is big enough to supply all of the data
procedure IrrSetMeshIndices(mesh: irr_mesh; frame: Integer; varindicies: Word); cdecl; external DLLNAME;

// get the number of vertices in the mesh buffer
function IrrGetMeshVertexCount(mesh: irr_mesh; frame: Integer): Integer; cdecl; external DLLNAME;

// copy the vertices of a mesh into the supplied buffer, the caller must ensure
// that the buffer is big enough to store the data
procedure IrrGetMeshVertices(mesh: irr_mesh; frame: Integer; varverticies: IRR_VERT); cdecl; external DLLNAME;

// copy the vertices in the supplied buffer into the mesh , the caller must
// ensure that the buffer is big enough to supply all of the data
procedure IrrSetMeshVertices(mesh: irr_mesh; frame: integer; varverticies: IRR_VERT); cdecl; external DLLNAME;

// gets the scenes root node, all scene nodes are children of this node
function IrrGetRootSceneNode(): irr_node; cdecl; external DLLNAME;

// add a loaded mesh to the scene: a new node
function IrrAddMeshToScene(mesh: irr_mesh): irr_node; cdecl; external DLLNAME;

// add a loaded mesh to the scene: an octtree. this is an efficient method
// for large complex meshes like BSP maps that occlude themselves a lot
function IrrAddMeshToSceneAsOcttree(mesh: irr_mesh): irr_node; cdecl; external DLLNAME;

// add a loaded mesh to the irrlicht scene manager using a static mesh buffer,
// tangent vertex information is added to allow the scene node to be used with
// normal and parallax mapping materials
function IrrAddStaticMeshForNormalMappingToScene(mesh: irr_mesh): irr_node; cdecl; external DLLNAME;


// load a scene created by irrEdit
procedure IrrLoadScene(filename: PChar); cdecl; external DLLNAME;

// save the current scene into a file that can be loaded by irrEdit
procedure IrrSaveScene(filename: PChar); cdecl; external DLLNAME;

// get a scene node based on its ID and returns null if no node is found
function IrrGetSceneNodeFromId(id: integer): irr_node; cdecl; external DLLNAME;

// get a node in the scene based upon its name and returns null is no node is found
function IrrGetSceneNodeFromName(name: PChar): irr_node; cdecl; external DLLNAME;


// adds a billboard to the scene this is a flat 3D textured sprite
function IrrAddBillBoardToScene(sizex, sizey: Single; x: single = 0; y: single = 0; z: single = 0): irr_node; cdecl; external DLLNAME;

// add a particle system to the irrlicht scene manager
Function IrrAddParticleSystemToScene(add_emitter: Integer; parent: irr_node = 0; id: Integer = -1; posX: Single = 0; posY: Single = 0; posZ: Single = 0; rotX: Single = 0; rotY: Single = 0; rotZ: Single = 0; scaleX: Single = 1; scaleY: Single = 1; scaleZ: Single = 1): irr_particle_system; cdecl; external DLLNAME;

// add a skybox to the irrlicht scene manager based on six pictures
function IrrAddSkyBoxToScene(up_texture, down_texture, left_texture, right_texture, front_texture, back_texture: irr_texture): irr_node; cdecl; external DLLNAME;

// add a skydome to the irrlicht scene manager
Function IrrAddSkyDomeToScene(the_texture: irr_texture; horizontal_res, vertical_res: Cardinal; texture_percentage, sphere_percentage: Double; radius: Double = 1000.0): irr_node; cdecl; external DLLNAME; 

// adds a test node to the scene, the node is a simple cube
function IrrAddTestSceneNode(): irr_node; cdecl; external DLLNAME;

// adds a simple cube node to the scene
function IrrAddCubeSceneNode(size: single): irr_node; cdecl; external DLLNAME;

// adds a simple sphere node to the scene
function IrrAddSphereSceneNode(redius: single; polyCount: integer): irr_node; cdecl; external DLLNAME;

// adds a simple sphere node to the scene
function IrrAddSphereSceneMesh(name: PChar; redius: single; polyCount: integer): irr_mesh; cdecl; external DLLNAME;

// add a scene node for rendering an animated water surface mesh
Function IrrAddWaterSurfaceSceneNode(mesh: irr_mesh; waveHeight: Single = 2.0; waveSpeed: Single = 300.0; waveLength: Single = 10.0; parent: irr_scene_node = 0; id: Integer = -1; positionX: Single = 0; positionY: Single = 0; positionZ: Single = 0; rotationX: Single = 0; rotationY: Single = 0; rotationZ: Single = 0; scaleX: Single = 1.0; scaleY: Single = 1.0; scaleZ: Single = 1.0): irr_node; cdecl; external DLLNAME;

// add a scene node for rendering an animated water surface mesh
Function IrrAddClouds(texture: irr_texture; lod, depth ,density: Cardinal): irr_node; cdecl; external DLLNAME;

// add a scene node for rendering a lens flare effect
Function IrrAddLensFlare(texture: irr_texture): irr_node; cdecl; external DLLNAME;

// add a scene node for rendering a patch of grass
Function IrrAddGrass(terrain: irr_terrain; x, y: integer; patchSize: integer; grassScale: single; maxDensity: Cardinal; dataPositionX, dataPositionY: integer; heightMap, textureMap, grassMap: irr_image; grassTexture: irr_texture): irr_node; cdecl; external DLLNAME;

// adds a zonemanagement object
function IrrAddZoneManager(nearDistance: single = 0; farDistance: single = 12000): irr_node; cdecl; external DLLNAME;

procedure IrrSetZoneManagerProperties(zoneNode: irr_node; nearDistance, farDistance: single; accumulateChildBoxes: Cardinal); cdecl; external DLLNAME;

procedure IrrSetZoneManagerBoundingBox(zoneNode: irr_node; x, y, z, boxWidth, boxHeight, boxDepth: single); cdecl; external DLLNAME;

procedure IrrSetZoneManagerAttachTerrain(zoneNode: irr_node; terrain: irr_terrain; structureMapFile, colorMapFile, detailMapFile: PChar; ImageX, ImageY, sliceSize: integer); cdecl; external DLLNAME;

// set grass density
procedure IrrSetGrassDensity(grass: irr_node; density: integer; distance: single); cdecl; external DLLNAME;

// set wind effect on grass
procedure IrrSetGrassWind(grass: irr_node; strength, resolution: single); cdecl; external DLLNAME;

function IrrGetGrassDrawCount(grass: irr_node): Cardinal; cdecl; external DLLNAME;

procedure IrrSetFlareScale(flare: irr_node; source, optics: single); cdecl; external DLLNAME;

// set the color of shadows in the scene
procedure IrrSetShadowColor(Alpha, R, G, B: Integer); cdecl; external DLLNAME;

// set the scene fog
procedure IrrSetFog(R, G, B, fogtype: Integer; fog_start, fog_end, density: single); cdecl; external DLLNAME;

// draw a 3D line into the display
procedure IrrDraw3DLine(xStart, yStart, zStart, xEnd, yEnd, zEnd: single; R, G, B: Cardinal); cdecl; external DLLNAME;

procedure IrrSetSkyDomeColor(dome: irr_node; horizontalRed, horizontalGreen, horizontalBlue, zenithRed, zenithGreen, zenithBlue: Cardinal); cdecl; external DLLNAME;

// apply a colored band to the skydome this can be used to represent a change in
// lighting at the horizon
procedure IrrSetSkyDomeColorBand(dome: irr_node; horizontalRed, horizontalGreen, horizontalBlue: Cardinal; bandVerticalPosition: integer; bandFade: single; addative: Cardinal); cdecl; external DLLNAME;

procedure IrrSetSkyDomeColorPoint(dome: irr_node; Red, Green, Blue: Cardinal; pointXPosition, pointYPosition, pointZPosition, pointRadius, pointFade: single; addative: Cardinal); cdecl; external DLLNAME;



//// ////////////////////////////////////////////////////////////////////////////
//// Scene Node Functions

// Get the number of materials:sociated with the node
function IrrGetMaterialCount(node: irr_node): Cardinal; cdecl; external DLLNAME;

// Get the material:sociated with the node at the particular index
function IrrGetMaterial(node: irr_node; index: Cardinal): irr_material; cdecl; external DLLNAME;

// apply an image file: a texturing material to a node
procedure IrrSetNodeMaterialTexture(node: irr_node; texture: irr_texture; material_index: integer); cdecl; external DLLNAME;

// set a material flag property on a node. the value can be either 0 or 1
procedure IrrSetNodeMaterialFlag(node: irr_node; flag: IRR_MATERIAL_FLAGS; value: Cardinal); cdecl; external DLLNAME;

// set the way the materials applied to the node are rendered
procedure IrrSetNodeMaterialType(node: irr_node; mat_type: IRR_MATERIAL_TYPES); cdecl; external DLLNAME;


// move a node in the scene to a new position
procedure IrrSetNodePosition(node: irr_node; X, Y, Z: single); cdecl; external DLLNAME;

// rotate a node in the scene
procedure IrrSetNodeRotation(node: irr_node; X ,Y, Z: single); cdecl; external DLLNAME;

// scale a node in the scene
procedure IrrSetNodeScale(node: irr_node; X, Y, Z: single); cdecl; external DLLNAME;

//render debugging information for nodes to the display
procedure IrrDebugDataVisible(node: irr_node; visible: Integer = $FFFFFFFF); cdecl; external DLLNAME;


// get the position of the node in the scene
procedure IrrGetNodePosition(node: irr_node; varX, varY, varZ: single); cdecl; external DLLNAME;

// get the absolute position of the node in the scene rather than its relative
// position in relation to its parent
procedure IrrGetNodeAbsolutePosition(node: irr_node; varX, varY, varZ: single); cdecl; external DLLNAME;

// get the rotation of the node in the scene
procedure IrrGetNodeRotation(node: irr_node; varX, varY, varZ: single); cdecl; external DLLNAME;


// set the portion of the animation sequence in the node that is to be played
procedure IrrSetNodeAnimationRange(node: irr_node; startFrame, iEndFrame: integer); cdecl; external DLLNAME;

// set the MD2 animation sequence to playback in the animation
procedure IrrPlayNodeMD2Animation(node: irr_node; sequence: IRR_MD2_ANIM_SEQUENCES); cdecl; external DLLNAME;

// set the speed of animation playback
procedure IrrSetNodeAnimationSpeed(node: irr_node; speed: integer); cdecl; external DLLNAME;

// get the current frame number being played in the animation
function IrrGetNodeAnimationFrame(node: irr_node): Cardinal; cdecl; external DLLNAME;

// set the current frame number being played in the animation
procedure IrrSetNodeAnimationFrame(node: irr_node; frame: integer); cdecl; external DLLNAME;

// set the time in seconds across which two animation frames are blended
procedure IrrSetTransitionTime(node: irr_node; speed: single); cdecl; external DLLNAME;

//Animates the mesh based on the position of the joints, this should be used at
//the end of any manual joint operations including blending and joints animated
//using IRR_JOINT_MODE_CONTROL and IrrSetNodeRotation on a bone node
procedure IrrAnimateJoints(node: irr_node); cdecl; external DLLNAME;

// Sets the animation mode of joints in a node
// IRR_JOINT_MODE_NONE will result in no animation of the model based on bones
// IRR_JOINT_MODE_READ will result in automatic animation based upon 
// IRR_JOINT_MODE_CONTROL will allow the position of the bones to be set through code
// When using the control mode IrrAnimateJoints must be called before IrrDrawScene
procedure IrrSetJointMode(node: irr_node; mode: Cardinal); cdecl; external DLLNAME;


// get a node that represents the position of a joint in a Milkshape skeleton
function IrrGetJointNode(node: irr_node; joint_name: PChar): irr_node; cdecl; external DLLNAME;

// get a node that represents the position of a joint in a Milkshape skeleton
// (note this function is now redundant please use IrrGetJointNode instead)
function IrrGetMS3DJointNode(node: irr_node; joint_name: PChar): irr_node; cdecl; external DLLNAME;

// get a node that represents the position of a joint in a Direct X skeleton
// (note this function is now redundant please use IrrGetJointNode instead)
function IrrGetDirectXJointNode(node: irr_node; joint_name: PChar): irr_node; cdecl; external DLLNAME;

// adds a node to another node: a child to a parent
procedure IrrAddChildToParent(child, parent: irr_node); cdecl; external DLLNAME;


// add a shadow to a node
procedure IrrAddNodeShadow(node: irr_node); cdecl; external DLLNAME;

// set whether a node is visible or not
procedure IrrSetNodeVisibility(node: irr_node; visible: integer); cdecl; external DLLNAME;

// delete this node from the scene
procedure IrrRemoveNode(node: irr_node); cdecl; external DLLNAME;

// Clears the entire scene, any references to nodes in the scene will become invalid
procedure IrrRemoveAllNodes(); cdecl; external DLLNAME;


// get the parent of this node, returns 0 if there is no parent
function IrrGetNodeParent(node: irr_node): irr_node; cdecl; external DLLNAME;

// set the parent of this node
procedure IrrSetNodeParent(node, parent: irr_node); cdecl; external DLLNAME;

// get the first child node of this node, returns 0 if there is no child
function IrrGetNodeFirstChild(node: irr_node; varposition: Pointer): irr_node; cdecl; external DLLNAME;

// get the next child node of this node, returns 0 if there is no child
function IrrGetNodeNextChild(node: irr_node; varposition: Pointer): irr_node; cdecl; external DLLNAME;

// returns true if this is the last child of the parent
function IrrIsNodeLastChild(node: irr_node; varposition: Pointer): irr_node; cdecl; external DLLNAME;

// get the ID of this node
function IrrGetNodeID(node: irr_node): integer; cdecl; external DLLNAME;

// set the ID of this node
procedure IrrSetNodeID(node: irr_node; id: integer); cdecl; external DLLNAME;

// get the name of this node
function IrrGetNodeName(node: irr_node): PChar; cdecl; external DLLNAME;

// set the name of this node
procedure IrrSetNodeName(node: irr_node; name: PChar); cdecl; external DLLNAME;



//// ////////////////////////////////////////////////////////////////////////////
//// Animator Functions

// add a collision animator to a node
function IrrAddCollisionAnimator(selector: irr_selector; node: irr_node; radiusx, radiusy, radiusz, gravityx, gravityy, gravityz, offsetx, offsety, offsetz: single): irr_animator; cdecl; external DLLNAME;

// add a deletion animator to a node
function IrrAddDeleteAnimator(node: irr_node; delete_after_n_milliseconds: integer): irr_animator; cdecl; external DLLNAME;

// add a fly-in-circle animator to a node
function IrrAddFlyCircleAnimator(node: irr_node; centre_x, centre_y, centre_z, radius, speed: single): irr_animator; cdecl; external DLLNAME;

// add a fly-straight animator to a node
function IrrAddFlyStraightAnimator(node: irr_node; start_x, start_y, start_z, end_x, end_y, end_z: single; time_to_complete: Cardinal; loop_path: integer): irr_animator; cdecl; external DLLNAME;

// add a rotation animator to a node
function IrrAddRotationAnimator(node: irr_node; x, y, z: single): irr_animator; cdecl; external DLLNAME;

// add a spline animator to a node
function IrrAddSplineAnimator(node: irr_node; array_size: integer; varx, vary, varz: single; time_to_start: integer; speed, tightness: single): irr_animator; cdecl; external DLLNAME;

// remove an animator from a node
procedure IrrRemoveAnimator(node: irr_node; node2: irr_animator); cdecl; external DLLNAME;



//// ////////////////////////////////////////////////////////////////////////////
//// Collision Functions

// gets a collision object from a complex mesh like a map
function IrrGetCollisionGroupFromComplexMesh(mesh: irr_mesh; node: irr_node): irr_selector; cdecl; external DLLNAME;

// creates a collision object from triangles in a node defined by its
// bounding box
function IrrGetCollisionGroupFromBox(node: irr_node): irr_selector; cdecl; external DLLNAME;

// creates a collision selector from a terrain node
function IrrGetCollisionGroupFromTerrain(node: irr_node; level_of_detail: integer): irr_selector; cdecl; external DLLNAME;

// creates a combined collision object that is a group of collision objects
// for example you could combine a map and a terrain
function IrrCreateCombinedCollisionGroup(): irr_selector; cdecl; external DLLNAME;

// adds a collision object to a combined collision object
procedure IrrAddCollisionGroupToCombination(combined_collision_group, collision_group: irr_selector); cdecl; external DLLNAME;

// remove all collision objects from the combined collision object
procedure IrrRemoveAllCollisionGroupsFromCombination(combined_collision_group: irr_selector); cdecl; external DLLNAME;

// remove a particular collision object from the combined collision object
procedure IrrRemoveCollisionGroupFromCombination(combined_collision_group, collision_group: irr_selector); cdecl; external DLLNAME;

// detect the collision point of a ray in the scene with a collision object if a
// collision was detected 1 is returned and vector collision contains the
// co-ordinates of the point of collision
function IrrGetCollisionPoint(varstart, varline_end: IRR_VECTOR; collision_group: irr_selector; varcollision_point: IRR_VECTOR): integer; cdecl; external DLLNAME;

// get a ray that goes from the specified camera and through the screen
// coordinates the information is coppied into the supplied start and end vectors
procedure IrrGetRayFromScreenCoordinates(screen_x, screen_y: integer; camera: irr_camera; varstart, varline_end: IRR_VECTOR); cdecl; external DLLNAME;

// a ray is cast through the camera and the nearest node that is hit by the ray
// is returned. if no node is hit zero is returned for the object
function IrrGetCollisionNodeFromCamera(camera: irr_camera): irr_node; cdecl; external DLLNAME;

// a ray is cast through the supplied coordinates and the nearest node that is
// hit by the ray is returned. if no node is hit zero is returned for the object
function IrrGetCollisionNodeFromRay(varstart, varline_end: IRR_VECTOR): irr_node; cdecl; external DLLNAME;

// a ray is cast through the supplied coordinates and the nearest node that is hit
// by the ray is returned. if no node is hit zero is returned for the object, only
// a procedureset of objects are tested, i.e. the children of the supplied node that
// match the supplied id. if the recurse option is enabled the entire tree of child
// objects connected to this node are tested
function IrrGetChildCollisionNodeFromRay(node: irr_node; idMask: integer; recurse: Cardinal; varstart, varline_end: IRR_VECTOR): irr_node; cdecl; external DLLNAME;

// the node and its children are recursively tested and the first node that contains
// the matched point is returned. if no node is hit zero is returned for the object,
// only a procedureset of objects are tested, i.e. the children of the supplied node that
// match the supplied id. if the recurse option is enabled the entire tree of child
// objects connected to this node are tested
function IrrGetChildCollisionNodeFromPoint(node: irr_node; idMask: integer; recurse: Cardinal; varpoint: IRR_VECTOR): irr_node; cdecl; external DLLNAME;

// a ray is cast through the screen at the specified co-ordinates and the nearest
// node that is hit by the ray is returned. if no node is hit zero is returned
// for the object
function IrrGetCollisionNodeFromScreenCoordinates(screen_x, screen_y: integer): irr_node; cdecl; external DLLNAME;

// screen co-ordinates are returned for the position of the specified 3d
// co-ordinates if an object were drawn at them on the screen, this is ideal for
// drawing 2D bitmaps or text around or on your 3D object on the screen
procedure IrrGetScreenCoordinatesFrom3DPosition(varscreen_x, varscreen_y: integer; at_position: IRR_VECTOR); cdecl; external DLLNAME;

// an all-inclusive collision routine for .irr scenes
procedure IrrSetupIrrSceneCollision(camera: irr_camera); cdecl; external DLLNAME;



//// ////////////////////////////////////////////////////////////////////////////
//// Camera Functions

// add a configurable first person perspective camera to the scene
Function IrrAddFPSCamera(parent: irr_scene_node = 0; rotateSpeed: Single = 100.0; moveSpeed: Single = 500.0; id: Integer = -1; keyMapArray: PSKeyMap = 0; keyMapSize: Integer = 0; noVericalMovement: Byte = 0; jumpSpeed: Single = 0.0): irr_camera; cdecl; external DLLNAME;

// add a regular camera
function IrrAddCamera(cameraX, cameraY, cameraZ, targetX, targetY, targetZ: single): irr_camera; cdecl; external DLLNAME;

// add a maya style camera to the scene
function IrrAddMayaCamera(parent: irr_scene_node = 0; rotateSpeed: Single = 1500.0; zoomSpeed: Single = 200.0; moveSpeed: Single = 1500.0): irr_camera; cdecl; external DLLNAME;

// reposition the target location of a camera
procedure IrrSetCameraTarget(camera: irr_camera; X, Y, Z: single); cdecl; external DLLNAME;

// get the current target location of a camera, the location is copied into the
// supplied variables
procedure IrrGetCameraTarget(camera: irr_camera; varX, varY, varZ: single); cdecl; external DLLNAME;

// set the up vector of a camera object, this controls the upward direction of
// the camera and allows you to roll it for free flight action
procedure IrrSetCameraUpDirection(camera: irr_camera; X, Y, Z: single); cdecl; external DLLNAME;

// get the vectors describing the camera direction useful after the camera has been revolved
procedure IrrGetCameraOrientation(camera: irr_camera; varX, varY, varZ: IRR_VECTOR); cdecl; external DLLNAME;

// set the distance at which the camera starts to clip polys
procedure IrrSetCameraClipDistance(camera: irr_camera; distance: single); cdecl; external DLLNAME;

// set the active camera in the scene
procedure IrrSetActiveCamera(camera: irr_camera); cdecl; external DLLNAME;

//sets the field of view (Default: PI / 2.5f). 
procedure IrrSetCameraFOV(camera: irr_camera; fov: single); cdecl; external DLLNAME;

//revolve the camera using quaternion calculations, this will help avoid gimbal
//lock:sociated with normal Rotations many thanks to RogerBorg for this
procedure IrrRevolveCamera(camera: irr_camera; yaw, pitch, roll, drive, strafe, elevate: single); cdecl; external DLLNAME;

procedure IrrSetCameraUpAtRightAngle(camera: irr_camera); cdecl; external DLLNAME;


//// ////////////////////////////////////////////////////////////////////////////
//// Lighting Functions

//add a light to the  scene
// default parent = 0
Function IrrAddLight(parent: irr_node; x, y, z, red, green, blue, size: Single): irr_node; cdecl; external DLLNAME;

// set the scene ambient lighting
procedure IrrSetAmbientLight(R, G, B: single); cdecl; external DLLNAME;


// Ambient color emitted by the light.
procedure IrrSetLightAmbientColor(light: irr_node; R, G, B: single); cdecl; external DLLNAME;

// Changes the light strength fading over distance. Good values for distance
// effects use ( 1.0, 0.0, 0.0) and simply add small values to the second and
// third element.
procedure IrrSetLightAttenuation(light: irr_node; constant, linear, quadratic: single); cdecl; external DLLNAME;

// Does the light cast shadows?
procedure IrrSetLightCastShadows(light: irr_node; cast_shadows: Cardinal); cdecl; external DLLNAME;

// Diffuse color emitted by the light.
procedure IrrSetLightDiffuseColor(light: irr_node; R, G, B: single); cdecl; external DLLNAME;

// The light strength//s decrease between Outer and Inner cone.
procedure IrrSetLightFalloff(light: irr_node; Falloff: single); cdecl; external DLLNAME;

// The angle of the spot//s inner cone. Ignored for other lights.
procedure IrrSetLightInnerCone(light: irr_node; InnerCone: single); cdecl; external DLLNAME;

// The angle of the spot//s outer cone. Ignored for other lights.
procedure IrrSetLightOuterCone(light: irr_node; OuterCone: single); cdecl; external DLLNAME;

// Radius of light. Everything within this radius be be lighted.
procedure IrrSetLightRadius(light: irr_node; Radius: single); cdecl; external DLLNAME;

// Specular color emitted by the light.
procedure IrrSetLightSpecularColor(light: irr_node; R, G, B: single); cdecl; external DLLNAME;

// Type of the light. Default: ELT_POINT.
procedure IrrSetLightType(light: irr_node; light_type: E_LIGHT_TYPE); cdecl; external DLLNAME;



//// ////////////////////////////////////////////////////////////////////////////
//// Terrain Functions

// create a terrain node from a highfield map
//function IrrAddTerrain"IrrAddTerrain" (path: PChar ): irr_terrain

// create a terrain node from a highfield map
function IrrAddSphericalTerrain(
       topPath, frontPath, backPath, leftPath, rightPath, bottomPath: PChar;
       xPosition: single = 0.0;
       yPosition: single = 0.0;
       zPosition: single = 0.0;
       xRotation: single = 0.0;
       yRotation: single = 0.0;
       zRotation: single = 0.0;
       xScale: single = 1.0;
       yScale: single = 1.0;
       zScale: single = 1.0;
       vertexAlpha: integer = 255;
       vertexRed: integer = 255;
       vertexGreen: integer = 255;
       vertexBlue: integer = 255;
       smoothing: integer = 0;
       spherical: integer = 0;
       maxLOD: integer = 5;
       patchSize: IRR_TERRAIN_PATCH_SIZE = ETPS_17 ): irr_terrain; cdecl; external DLLNAME;

// create a tiled terrain node from a highfield map
function IrrAddTiledTerrain(
       heightMapPath, texturePath: PChar;
       followNode: irr_node;
       terrainWidth: integer = 1000;
       terrainHeight: integer = 1000;
       tileSize: single = 1.0;
       meshSize: integer = 100;
       heightmapScale: single = 10.0;
       smoothing: integer = 0.0;
       materialType: IRR_MATERIAL_TYPES = IRR_EMT_DETAIL_MAP ): irr_node; cdecl; external DLLNAME;

// create a terrain node from a highfield map
function IrrAddTerrain(
       path: PChar;
       xPosition: single = 0.0;
       yPosition: single = 0.0;
       zPosition: single = 0.0;
       xRotation: single = 0.0;
       yRotation: single = 0.0;
       zRotation: single = 0.0;
       xScale: single = 1.0;
       yScale: single = 1.0;
       zScale: single = 1.0;
       vertexAlpha: integer = 255;
       vertexRed: integer = 255;
       vertexGreen: integer = 255;
       vertexBlue: integer = 255;
       smoothing: integer = 0;
       maxLOD: integer = 5;
       patchSize: IRR_TERRAIN_PATCH_SIZE = ETPS_17 ): irr_terrain; cdecl; external DLLNAME;


// scale the texture on a terrain node
procedure IrrScaleTexture(terrain: irr_terrain; X, Y: single); cdecl; external DLLNAME;

// set the textures on all six faces of a spherical terrain
procedure IrrSetSphericalTerrainTexture(terrain, topTexture, frontTexture, backTexture, leftTexture, rightTexture, bottomTexture: irr_texture; materialIndex: Cardinal); cdecl; external DLLNAME;
        
// load the vertex colors on all six faces of a spherical terrain from images
procedure IrrLoadSphericalTerrainVertexColor(terrain: irr_terrain; topImage, frontImage, backImage, leftImage, rightImage, bottomImage: irr_image); cdecl; external DLLNAME;

// scale the texture on a spherical terrain node
procedure IrrScaleSphericalTexture(terrain: irr_terrain; X, Y: single); cdecl; external DLLNAME;

// get the height of a co-ordinate on a tiled terrain node
function IrrGetTerrainHeight(terrain: irr_terrain; X, Z: single): single; cdecl; external DLLNAME;

// get the height of a co-ordinate on a tiled terrain node
function IrrGetTiledTerrainHeight(terrain: irr_terrain; X, Z: single): single; cdecl; external DLLNAME;

// get the surface position of a co-ordinate on a spherical terrain node
procedure IrrGetSphericalTerrainSurfacePosition(terrain: irr_terrain; face: integer; logicalX, logicalZ, varX, varY, varZ: single); cdecl; external DLLNAME;

// get the surface angle of a co-ordinate on a spherical terrain node
procedure IrrGetSphericalTerrainSurfacePositionAndAngle(terrain: irr_terrain; face: integer; logicalX, logicalZ, varpositionX, varpositionY, varpositionZ, varangleX, varangleY, varangleZ: single); cdecl; external DLLNAME;

// get the logical position on a surface based on a real co-ordinate
procedure IrrGetSphericalTerrainLogicalSurfacePosition(terrain: irr_terrain; X, Y, Z: single; varface: integer; varlogicalX, varlogicalZ: single); cdecl; external DLLNAME;

// create a terrain node from a highfield map
function IrrAddTerrainTile(
       image: irr_image;
       tileSize: integer = 256;
       dataX: integer = 0;
       dataY: integer = 0;
       xPosition: single = 0.0;
       yPosition: single = 0.0;
       zPosition: single = 0.0;
       xRotation: single = 0.0;
       yRotation: single = 0.0;
       zRotation: single = 0.0;
       xScale: single = 1.0;
       yScale: single = 1.0;
       zScale: single = 1.0;
       smoothing: integer = 1;
       maxLOD: integer = 5;
       patchSize: IRR_TERRAIN_PATCH_SIZE = ETPS_17 ): irr_terrain; cdecl; external DLLNAME;

// get the height of a point on a terrain tile node
function IrrGetTerrainTileHeight(terrain: irr_terrain; X, Z: single): single; cdecl; external DLLNAME;

// scale the texture detail on a tiled terrain node
procedure IrrScaleTileTexture(terrain: irr_terrain; X, Y: single); cdecl; external DLLNAME;

// attach a tile onto this tile
procedure IrrAttachTile(terrain, neighbour: irr_terrain; Edge: Cardinal); cdecl; external DLLNAME;

// load the vertex color information of a tile
procedure IrrSetTileColor(terrain: irr_terrain; image: irr_image; dataX: integer = 0; dataY: integer = 0); cdecl; external DLLNAME;

// load the height and UV data from an image
procedure IrrSetTileStructure(terrain: irr_terrain; image: irr_image; dataX: integer = 0; dataY: integer = 0 ); cdecl; external DLLNAME;



//// ////////////////////////////////////////////////////////////////////////////
//// Particle Functions

// set the size of a particle in the particle system
procedure IrrSetParticleSize(particle_system: irr_particle_system; X, Y: single); cdecl; external DLLNAME;

// create an emitter that can be added to a particle system
function IrrAddParticleEmitter(particle_system: irr_particle_system; settings: IRR_PARTICLE_EMITTER): irr_emitter; cdecl; external DLLNAME;

// Creates a particle emitter for an animated mesh scene node
function IrrAddAnimatedMeshSceneNodeEmitter(particle_system: irr_particle_system; node: irr_node; use_normal_direction: Cardinal; normal_direction_modifier: single; emit_from_every_vertex: integer; settings: IRR_PARTICLE_EMITTER): irr_emitter; cdecl; external DLLNAME;


// Add an affector to the particle system to fade the particles out
function IrrAddFadeOutParticleAffector(particle_system: irr_particle_system; fade_speed, fade_to_red, fade_to_green, fade_to_blue: Cardinal): irr_affector; cdecl; external DLLNAME;

// Add an affector to the particle system to alter their position with gravity
function IrrAddGravityParticleAffector(particle_system: irr_particle_system; x, y, z: Single; timeForceLost: Cardinal = 1000): irr_affector; cdecl; external DLLNAME;

// Creates a point attraction affector. This affector modifies the positions of
// the particles and attracts them to a specified point at a specified speed per
// second.
function IrrAddParticleAttractionAffector(particle_system: irr_particle_system; x, y, z: Single; speed: Single = 1.0; attract: Cardinal = 1; affectX: Cardinal = 1; affectY: Cardinal = 1; affectZ: Cardinal = 1): irr_affector; cdecl; external DLLNAME;

// Creates a rotation affector. This affector modifies the positions of the
// particles and attracts them to a specified point at a specified speed per second.
function IrrAddRotationAffector(particle_system: irr_particle_system; Speed_X, Speed_Y, Speed_Z, pivot_X, pivot_Y, pivot_Z: Single): irr_affector; cdecl; external DLLNAME;

// Creates a Stop Particle Affector that automatically stops the emmision of
// particles from the specified emitter after a period of time
function IrrAddStopParticleAffector(particle_system: irr_particle_system; run_time: Cardinal; target_emitter: irr_emitter): irr_affector; cdecl; external DLLNAME;

// adds a push affector with a limited range of effect
function IrrAddParticlePushAffector(particle_system: irr_particle_system; x, y, z, speedX, speedY, speedZ, far, near, column: single; radial: integer): irr_affector; cdecl; external DLLNAME;

// adds a color affector to change the color of particles over time through an
// array of user defined colors
function IrrAddColorMorphAffector(particle_system: irr_particle_system; num: Cardinal; particlecolors, particletimes: PCardinal; smooth: Cardinal): irr_affector; cdecl; external DLLNAME;

// adds a spline affector to move particles along the path of a user defined
// spline
function IrrAddSplineAffector(particle_system: irr_particle_system; VertexCount: Cardinal; verticies: PIRR_VERT; speed, tightness, attraction: single; deleteAtEnd: Cardinal): irr_affector; cdecl; external DLLNAME;

// remove all effectors from this particle system
procedure IrrRemoveAffectors(particle_system: irr_particle_system); cdecl; external DLLNAME;


// Set direction the emitter emits particles.
procedure IrrSetParticleEmitterDirection(particle_system: irr_emitter; x, y, z: single); cdecl; external DLLNAME;

// Set minimum number of particles the emitter emits per second.
procedure IrrSetParticleEmitterMinParticlesPerSecond(particle_system: irr_emitter; minPPS: Cardinal); cdecl; external DLLNAME;

// Set maximum number of particles the emitter emits per second.
procedure IrrSetParticleEmitterMaxParticlesPerSecond(particle_system: irr_emitter; maxPPS: Cardinal); cdecl; external DLLNAME;

// Set minimum starting color for particles.
procedure IrrSetParticleEmitterMinStartColor(particle_system: irr_emitter; Red, Green, Blue: Cardinal); cdecl; external DLLNAME;

// Set maximum starting color for particles.
procedure IrrSetParticleEmitterMaxStartColor(particle_system: irr_emitter; Red, Green, Blue: Cardinal); cdecl; external DLLNAME;


// Enable or disable an affector
procedure IrrSetParticleAffectorEnable(affector: irr_affector; enable: Cardinal); cdecl; external DLLNAME;

// Alter the fadeout affector changing the fade out time
procedure IrrSetFadeOutParticleAffectorTime(affector: irr_affector; fade_speed: single); cdecl; external DLLNAME;

// Alter the fadeout affector changing the target color
procedure IrrSetFadeOutParticleAffectorTargetColor(affector: irr_affector; fade_to_red, fade_to_green, fade_to_blue: Cardinal); cdecl; external DLLNAME;

// Alter the direction and force of gravity
procedure IrrSetGravityParticleAffectorDirection(affector: irr_affector; x, y, z: single); cdecl; external DLLNAME;

// Set the time in milliseconds when the gravity force is totally lost and the particle does not move any more
procedure IrrSetGravityParticleAffectorTimeForceLost(affector: irr_affector; time_Force_Lost: single); cdecl; external DLLNAME;

// Set whether or not this will affect particles in the X direction.
procedure IrrSetParticleAttractionAffectorAffectX(affector: irr_affector; affect_x: Cardinal); cdecl; external DLLNAME;

// Set whether or not this will affect particles in the Y direction.
procedure IrrSetParticleAttractionAffectorAffectY(affector: irr_affector; affect_y: Cardinal); cdecl; external DLLNAME;

// Set whether or not this will affect particles in the Z direction.
procedure IrrSetParticleAttractionAffectorAffectZ(affector: irr_affector; affect_z: Cardinal); cdecl; external DLLNAME;

// Set whether or not the particles are attracting or detracting.
procedure IrrSetParticleAttractionAffectorAttract(affector: irr_affector; attract: Cardinal); cdecl; external DLLNAME;
 
// Set the point that particles will attract to. 
procedure IrrSetParticleAttractionAffectorPoint(affector: irr_affector; x, y, z: single); cdecl; external DLLNAME;

// Set the point that particles will rotate about.
procedure IrrSetRotationAffectorPivotPoint(affector: irr_affector; x, y, z: single); cdecl; external DLLNAME;

// Set the furthest distance for the push particle effect
procedure IrrSetFurthestDistanceOfEffect(affector: irr_affector; NewDistance: single); cdecl; external DLLNAME;

// Set the nearest distance for the push particle effect
procedure IrrSetNearestDistanceOfEffect(affector: irr_affector; NewDistance: single); cdecl; external DLLNAME;

// Set the column width of the push particle effect
procedure IrrSetColumnDistanceOfEffect(affector: irr_affector; NewDistance: single); cdecl; external DLLNAME;

// Set the center of the push particle effect
procedure IrrSetCenterOfEffect(affector: irr_affector; x, y, z: single); cdecl; external DLLNAME;

// Set the strength of the push particle effect
procedure IrrSetStrengthOfEffect(affector: irr_affector; x, y, z: single); cdecl; external DLLNAME;

// Set the radial effect of the push particle effect
procedure IrrSetRadialEffect(affector: irr_affector; IsRadial: integer); cdecl; external DLLNAME;


/////////////////////////////////////////////////////////////////////////////
// GUI Functions

// add a static text object to the gui display
procedure IrrAddStaticText(Text: WideString; TopX, TopY, BotX, BotY: Integer; Border, WordWrap: Cardinal); cdecl; external DLLNAME;

  
implementation

function IrrMakeARGB(Alpha, Red, Green, Blue: Integer): Cardinal; 
begin
  Result := (Alpha SHL 24) + (Red SHL 16) + (Green SHL 8) + Blue;
end;

procedure IrrHideMouse();
begin
 IrrDisplayMouse(0);
end;

procedure IrrShowMouse(); 
begin
  IrrDisplayMouse(1)
end;
end.
